@@ -1,5 +1,5 @@
 
-/*  $Header: /cvs/gnome/gcalctool/gcalctool/calctool.c,v 1.88 2005/02/07 16:38:58 richb Exp $
+/*  $Header: /cvs/gnome/gcalctool/gcalctool/calctool.c,v 1.91 2005/05/16 16:15:22 richb Exp $
  *
  *  Copyright (c) 1987-2005 Sun Microsystems, Inc. All Rights Reserved.
  *           
@@ -50,7 +50,9 @@ char *base_desc[]  = {         /* Tooltips for each base value. */
 
 char *calc_res[] = {
     "accuracy", "base", "display", "modetype", "showregisters", "trigtype",
-    "showzeroes", "showthousands", "syntax"
+    "showzeroes", "showthousands", "syntax",
+    "register0", "register1", "register2", "register3", "register4",
+    "register5", "register6", "register7", "register8", "register9"
 };
 
 char *dtype_str[] = {          /* Strings for each display mode value. */
@@ -105,7 +107,7 @@ Vars v;            /* Calctool variables and options. */
  *
  *           |  a b c d e f g h i j k l m n o p q r s t u v w x y z
  *-----------+-----------------------------------------------------
- *  Lower:   |  a b c d e f     i     l m n   p   r s t u v     y
+ *  Lower:   |  a b c d e f     i     l m n   p   r s t u v   x
  *  Upper:   |  A   C D E F G     J K L   N   P   R S T       X Y
  *  Numeric: |  0 1 2 3 4 5 6 7 8 9
  *  Other:   |  @ . + - * / = % ( ) # < > [ ] { } | & ~ ^ ? ! :
@@ -904,11 +906,11 @@ struct button s_buttons[S_NOBUTTONS] = {   /* Scientific mode button values. */
 },       
 {        
     N_("<i>x</i><sup><i>y</i></sup>"),
-    N_("Raise displayed value to the power of y [y]"),
+    N_("Raise displayed value to the power of y [^]"),
     N_("X to the y"),
-    { 0, 0 },   
-    { GDK_y, 0 },
-    'y',
+    { GDK_SHIFT_MASK, GDK_SHIFT_MASK,  0 },
+    { GDK_caret,      GDK_asciicircum, 0 },
+    '^',
     M_NONE,
     do_calc,
     N_("^"),
@@ -1111,11 +1113,11 @@ struct button s_buttons[S_NOBUTTONS] = {   /* Scientific mode button values. */
 },
 {
     N_("Xor"),
-    N_("Bitwise XOR [^]"),
+    N_("Bitwise XOR [x]"),
     NULL,
-    { GDK_SHIFT_MASK, GDK_SHIFT_MASK,  0 },
-    { GDK_caret,      GDK_asciicircum, 0 },
-    '^',
+    { 0, 0 },   
+    { GDK_x, 0 },
+    'x',
     M_NONE,
     do_calc,
     N_(" Xor "),
@@ -1208,6 +1210,8 @@ do_calctool(int argc, char **argv)
 
     show_display(v->MPdisp_val);     /* Output in correct display mode. */
 
+    memset(&(v->e), 0, sizeof(struct exprm_state)); /* clear expression mode state */
+
     start_tool();                    /* Display the calculator. */
 }
 
