@@ -1,5 +1,5 @@
 
-/*  $Header: /cvs/gnome/gcalctool/gcalctool/calctool.h,v 1.67 2005/02/07 16:38:58 richb Exp $
+/*  $Header: /cvs/gnome/gcalctool/gcalctool/calctool.h,v 1.71 2005/04/11 17:38:33 richb Exp $
  *
  *  Copyright (c) 1987-2005 Sun Microsystems, Inc. All Rights Reserved.
  *           
@@ -74,12 +74,14 @@ enum num_type { ENG, FIX, SCI };            /* Number display mode. */
 
 /* Resources. */
 enum res_type { R_ACCURACY, R_BASE, R_DISPLAY, R_MODE, R_REGS, R_TRIG,
-                R_ZEROES,   R_TSEP, R_SYNTAX
+                R_ZEROES,   R_TSEP, R_SYNTAX, 
+                R_REG0,     R_REG1, R_REG2,    R_REG3, R_REG4,
+                R_REG5,     R_REG6, R_REG7,    R_REG8, R_REG9
 };
 
 enum trig_type { DEG, GRAD, RAD };          /* Trigonometric types. */
 
-enum trig_func { SIN=1, COS=2, TAN=4 };
+enum trig_func {SIN=0, COS=1, TAN=2};       
 
 /* Abbreviations for the gcalctool keyboard and menu equivalents. */
 
@@ -385,7 +387,7 @@ char *gc_strdup(char *str);
 const char *get_radix();
 char *get_resource(enum res_type);
 const char *get_tsep();
-char *make_number(int *, int, BOOLEAN, BOOLEAN);
+char *make_number(int *, int, BOOLEAN);
 char *set_bool(int);
 
 unsigned short *get_but_data();
@@ -397,6 +399,7 @@ int do_rcl_reg(int reg, int value[MP_SIZE]);
 int do_sto_reg(int reg, int value[MP_SIZE]);
 int do_tfunc(int s[MP_SIZE], int t[MP_SIZE], enum trig_func tfunc);
 int get_menu_entry(enum menu_type, int);
+int key_equal(struct button *, struct button);
 int main(int, char **);
 
 void beep();
@@ -470,6 +473,7 @@ void set_mode(enum mode_type);
 void set_title(enum fcp_type, char *);
 void show_display(int *);
 void show_error(char *);
+void show_menu();
 void srand48();
 void start_tool();
 void str_replace(char **, char *, char *);
