@@ -1,5 +1,5 @@
 
-/*  $Header: /cvs/gnome/gcalctool/gcalctool/gtk.c,v 1.156 2005/02/07 16:38:58 richb Exp $
+/*  $Header: /cvs/gnome/gcalctool/gcalctool/gtk.c,v 1.174 2005/05/02 16:03:07 richb Exp $
  *
  *  Copyright (c) 1987-2005 Sun Microsystems, Inc. All Rights Reserved.
  *           
@@ -27,7 +27,6 @@
 #include <sys/param.h>
 #include <sys/stat.h>
 #include <netdb.h>
-#include <pwd.h>
 #include "calctool.h"
 #include "extern.h"
 #include "dsdefs.h"
@@ -36,7 +35,9 @@
 #include <gtk/gtk.h>
 #include <gdk/gdkx.h>
 #include <gdk/gdkkeysyms.h>
+#ifndef DISABLE_GNOME
 #include <gnome.h>
+#endif /*DISABLE_GNOME*/
 #include <gconf/gconf-client.h>
 #include "ce_parser.h"
 
@@ -66,6 +67,14 @@
 #define BUT_CLR_ADV   X->adv_buttons[7]        /* Clr - in "Advanced" mode */
 
 #define BUT_ACC   X->sci_buttons[7]        /* a */
+#define BUT_CON   X->sci_buttons[8]        /* # */
+#define BUT_EXCH  X->adv_buttons[31]       /* e */
+#define BUT_FUN   X->sci_buttons[9]        /* f */
+#define BUT_LSFT  X->sci_buttons[0]        /* < */
+#define BUT_RCL   X->adv_buttons[23]       /* R */
+#define BUT_RSFT  X->sci_buttons[1]        /* > */
+#define BUT_STO   X->adv_buttons[15]       /* S */
+
 #define BUT_A     X->sci_buttons[24]       /* A */
 #define BUT_B     X->sci_buttons[25]       /* B */
 #define BUT_C     X->sci_buttons[26]       /* C */
@@ -91,7 +100,6 @@ struct Xobject {               /* Gtk+/Xlib graphics object. */
     GtkWidget *kframe;                 /* Main window. */
     GtkWidget *kvbox;
     GtkWidget *ktable;
-    GdkPixbuf *icon;                   /* Main window icon. */
     GtkWidget *menubar;
     GtkWidget *mode_panel;
 
@@ -169,7 +177,9 @@ static void cell_edited(GtkCellRendererText *,
                         const gchar *, const gchar *, gpointer);
 static void create_con_fun_menu(enum menu_type);
 static void create_menu_item_with_markup(char *, int, int);
+#ifndef DISABLE_GNOME
 static void die_cb(GnomeClient *client, gpointer data);
+#endif /*DISABLE_GNOME*/
 static void disp_cb(GtkToggleButton *, gpointer);
 static void hyp_cb(GtkToggleButton *, gpointer);
 static void inv_cb(GtkToggleButton *, gpointer);
@@ -186,12 +196,12 @@ static void quit_cb(GtkWidget *, gpointer);
 static void reset_mode_values(enum mode_type);
 static void set_accuracy_toggle(int val);
 static void set_button_state(GtkWidget *, int);
-static void set_gcalctool_icon(void);
 static void set_item(enum item_type itemtype, int val);
 static void set_memory_toggle(int);
 static void set_show_tsep_toggle(int);
 static void set_show_zeroes_toggle(int);
 static void show_ascii_frame();
+static void show_menu_for_button(GtkWidget *, GdkEventKey *event);
 static void show_precision_frame();
 static void spframe_cancel_cb(GtkButton *, gpointer);
 static void spframe_ok_cb(GtkButton *, gpointer);
@@ -208,76 +218,79 @@ static GSList *list;
 static XVars X;
 
 /* Menubar menus. */
+/* Use NULL to let the STOCK label values or keybindings show through */
 
-static GtkActionEntry entries[] = {
+static const GtkActionEntry entries[] = {
     { "CalculatorMenu", NULL, N_("_Calculator") },
     { "EditMenu",       NULL, N_("_Edit") },
     { "ViewMenu",       NULL, N_("_View") },
     { "HelpMenu",       NULL, N_("_Help") },
 
-    { "Quit", GTK_STOCK_QUIT, N_("_Quit"), "<control>Q",
+    { "Quit", GTK_STOCK_QUIT, NULL, "<control>Q",
       N_("Quit the calculator"), G_CALLBACK(mb_proc) },
 
-    { "Copy", GTK_STOCK_COPY, N_("_Copy"), NULL,
+    { "Copy", GTK_STOCK_COPY, NULL, NULL,
       N_("Copy selection"), G_CALLBACK(mb_proc) },
-    { "Paste", GTK_STOCK_PASTE, N_("_Paste"), NULL,
+    { "Paste", GTK_STOCK_PASTE, NULL, NULL,
       N_("Paste selection"), G_CALLBACK(mb_proc) },
     { "Insert", NULL, N_("_Insert ASCII Value..."), "<control>I",
       N_("Insert ASCII value"), G_CALLBACK(mb_proc) },
 
-    { "Contents", GTK_STOCK_HELP, N_("_Contents"), "F1",
+#ifndef DISABLE_GNOME
+    { "Contents", GTK_STOCK_HELP, NULL, "F1",
       N_("Show help contents"), G_CALLBACK(mb_proc) },
-    { "About", GTK_STOCK_ABOUT, N_("_About"), NULL,
+#endif /*DISABLE_GNOME*/
+    { "About", GTK_STOCK_ABOUT, NULL, NULL,
       N_("Show the About Gcalctool dialog"), G_CALLBACK(about_cb) },
 
-    { "LSPlaces1",  NULL, N_("1 place"),   NULL, 
+    { "LSPlaces1",  NULL, N_("_1 place"),   NULL,
       N_("1 place"),   G_CALLBACK(mb_proc) },
-    { "LSPlaces2",  NULL, N_("2 places"),  NULL, 
+    { "LSPlaces2",  NULL, N_("_2 places"),  NULL,
       N_("2 places"),  G_CALLBACK(mb_proc) },
-    { "LSPlaces3",  NULL, N_("3 places"),  NULL, 
+    { "LSPlaces3",  NULL, N_("_3 places"),  NULL,
       N_("3 places"),  G_CALLBACK(mb_proc) },
-    { "LSPlaces4",  NULL, N_("4 places"),  NULL, 
+    { "LSPlaces4",  NULL, N_("_4 places"),  NULL,
       N_("4 places"),  G_CALLBACK(mb_proc) },
-    { "LSPlaces5",  NULL, N_("5 places"),  NULL, 
+    { "LSPlaces5",  NULL, N_("_5 places"),  NULL, 
       N_("5 places"),  G_CALLBACK(mb_proc) },
-    { "LSPlaces6",  NULL, N_("6 places"),  NULL, 
+    { "LSPlaces6",  NULL, N_("_6 places"),  NULL, 
       N_("6 places"),  G_CALLBACK(mb_proc) },
-    { "LSPlaces7",  NULL, N_("7 places"),  NULL, 
+    { "LSPlaces7",  NULL, N_("_7 places"),  NULL, 
       N_("7 places"),  G_CALLBACK(mb_proc) },
-    { "LSPlaces8",  NULL, N_("8 places"),  NULL, 
+    { "LSPlaces8",  NULL, N_("_8 places"),  NULL, 
       N_("8 places"),  G_CALLBACK(mb_proc) },
-    { "LSPlaces9",  NULL, N_("9 places"),  NULL, 
+    { "LSPlaces9",  NULL, N_("_9 places"),  NULL, 
       N_("9 places"),  G_CALLBACK(mb_proc) },
-    { "LSPlaces10", NULL, N_("10 places"), NULL, 
+    { "LSPlaces10", NULL, N_("10 places"),  NULL,
       N_("10 places"), G_CALLBACK(mb_proc) },
-    { "LSPlaces11", NULL, N_("11 places"), NULL, 
+    { "LSPlaces11", NULL, N_("11 places"),  NULL,
       N_("11 places"), G_CALLBACK(mb_proc) },
-    { "LSPlaces12", NULL, N_("12 places"), NULL, 
+    { "LSPlaces12", NULL, N_("12 places"),  NULL,
       N_("12 places"), G_CALLBACK(mb_proc) },
-    { "LSPlaces13", NULL, N_("13 places"), NULL, 
+    { "LSPlaces13", NULL, N_("13 places"),  NULL,
       N_("13 places"), G_CALLBACK(mb_proc) },
-    { "LSPlaces14", NULL, N_("14 places"), NULL, 
+    { "LSPlaces14", NULL, N_("14 places"),  NULL,
       N_("14 places"), G_CALLBACK(mb_proc) },
-    { "LSPlaces15", NULL, N_("15 places"), NULL, 
+    { "LSPlaces15", NULL, N_("15 places"),  NULL,
       N_("15 places"), G_CALLBACK(mb_proc) },
 
-    { "RSPlaces1",  NULL, N_("1 place"),   NULL, 
+    { "RSPlaces1",  NULL, N_("_1 place"),   NULL, 
       N_("1 place"),   G_CALLBACK(mb_proc) },
-    { "RSPlaces2",  NULL, N_("2 places"),  NULL, 
+    { "RSPlaces2",  NULL, N_("_2 places"),  NULL, 
       N_("2 places"),  G_CALLBACK(mb_proc) },
-    { "RSPlaces3",  NULL, N_("3 places"),  NULL, 
+    { "RSPlaces3",  NULL, N_("_3 places"),  NULL, 
       N_("3 places"),  G_CALLBACK(mb_proc) },
-    { "RSPlaces4",  NULL, N_("4 places"),  NULL, 
+    { "RSPlaces4",  NULL, N_("_4 places"),  NULL, 
       N_("4 places"),  G_CALLBACK(mb_proc) },
-    { "RSPlaces5",  NULL, N_("5 places"),  NULL, 
+    { "RSPlaces5",  NULL, N_("_5 places"),  NULL, 
       N_("5 places"),  G_CALLBACK(mb_proc) },
-    { "RSPlaces6",  NULL, N_("6 places"),  NULL, 
+    { "RSPlaces6",  NULL, N_("_6 places"),  NULL, 
       N_("6 places"),  G_CALLBACK(mb_proc) },
-    { "RSPlaces7",  NULL, N_("7 places"),  NULL, 
+    { "RSPlaces7",  NULL, N_("_7 places"),  NULL, 
       N_("7 places"),  G_CALLBACK(mb_proc) },
-    { "RSPlaces8",  NULL, N_("8 places"),  NULL, 
+    { "RSPlaces8",  NULL, N_("_8 places"),  NULL, 
       N_("8 places"),  G_CALLBACK(mb_proc) },
-    { "RSPlaces9",  NULL, N_("9 places"),  NULL, 
+    { "RSPlaces9",  NULL, N_("_9 places"),  NULL, 
       N_("9 places"),  G_CALLBACK(mb_proc) },
     { "RSPlaces10", NULL, N_("10 places"), NULL, 
       N_("10 places"), G_CALLBACK(mb_proc) },
@@ -292,9 +305,8 @@ static GtkActionEntry entries[] = {
     { "RSPlaces15", NULL, N_("15 places"), NULL, 
       N_("15 places"), G_CALLBACK(mb_proc) },
 };
-static guint n_entries = G_N_ELEMENTS(entries);
 
-static GtkToggleActionEntry toggle_entries[] = {
+static const GtkToggleActionEntry toggle_entries[] = {
     { "Trailing",  NULL, N_("Show _Trailing Zeroes"),     "<control>T",
       N_("Show trailing zeroes"),     G_CALLBACK(mstz_proc), FALSE },
     { "Thousands", NULL, N_("Show T_housands Separator"), "<control>K",
@@ -306,9 +318,8 @@ static GtkToggleActionEntry toggle_entries[] = {
     { "ArithmeticPrecedence", NULL, N_("_Use Arithmetic Precedence"), "<control>U",
       N_("Use Arithmetic Precedence"), G_CALLBACK(mb_proc), FALSE },
 };
-static guint n_toggle_entries = G_N_ELEMENTS(toggle_entries);
 
-static GtkRadioActionEntry acc_radio_entries[] = {
+static const GtkRadioActionEntry acc_radio_entries[] = {
   { "SP0", NULL, N_("_0 significant places"), "<control>0", 
     N_("0 significant places"), '0' },
   { "SP1", NULL, N_("_1 significant place"),  "<control>1", 
@@ -332,9 +343,8 @@ static GtkRadioActionEntry acc_radio_entries[] = {
   { "SPOther", NULL, N_("_Other (10) ..."), "<control>O",
     N_("Set other precision"), 'O' },
 };
-static guint n_acc_radio_entries = G_N_ELEMENTS(acc_radio_entries);
 
-static GtkRadioActionEntry base_radio_entries[] = {
+static const GtkRadioActionEntry base_radio_entries[] = {
   { "Basic",      NULL, N_("_Basic"),      "<control>B", 
     N_("Basic"),      M_BASIC },
   { "Advanced",   NULL, N_("_Advanced"),   "<control>A",
@@ -345,9 +355,7 @@ static GtkRadioActionEntry base_radio_entries[] = {
     N_("Scientific"), M_SCI },
 };
 
-static guint n_base_radio_entries = G_N_ELEMENTS(base_radio_entries);
-
-static const gchar *ui_info =
+static const gchar ui_info[] =
 "<ui>"
 "  <menubar name='MenuBar'>"
 "    <menu action='CalculatorMenu'>"
@@ -373,7 +381,9 @@ static const gchar *ui_info =
 "      <menuitem action='ArithmeticPrecedence'/>"
 "    </menu>"
 "    <menu action='HelpMenu'>"
+#ifndef DISABLE_GNOME
 "      <menuitem action='Contents'/>"
+#endif /*DISABLE_GNOME*/
 "      <menuitem action='About'/>"
 "    </menu>"
 "  </menubar>"
@@ -432,9 +442,9 @@ static const gchar *ui_info =
 int
 main(int argc, char **argv)
 {
-    char name[MAXLINE];          /* Full name of users .gcalctoolrc file. */
-    struct passwd *entry;
+#ifndef DISABLE_GNOME
     GnomeClient  *client;
+#endif /*DISABLE_GNOME*/
 
     v = (Vars)  LINT_CAST(calloc(1, sizeof(struct calcVars)));
     X = (XVars) LINT_CAST(calloc(1, sizeof(struct Xobject)));
@@ -443,28 +453,29 @@ main(int argc, char **argv)
     bind_textdomain_codeset(GETTEXT_PACKAGE, "UTF-8");
     textdomain(GETTEXT_PACKAGE);
 
+#ifndef DISABLE_GNOME
     gnome_program_init("gcalctool", VERSION, LIBGNOMEUI_MODULE, argc, argv,
                         NULL, NULL, NULL);
 
-    X->lnp = get_localized_numeric_point();
-
     /* Connect to the die signal. */
     client = gnome_master_client();
     g_signal_connect(client, "die", G_CALLBACK(die_cb), NULL);
+#else
+    gtk_init(&argc, &argv);
+#endif /*DISABLE_GNOME*/
+
+    X->lnp = get_localized_numeric_point();
 
     gtk_rc_get_default_files();
-    if ((v->home = getenv("HOME")) == NULL) {
-        if ((entry = getpwuid(getuid())) != NULL) {
-            v->home = entry->pw_dir;
-        }
-    }
-    SPRINTF(name, "%s/%s", v->home, RCNAME);
-    gtk_rc_parse(name);
+
+    v->home = (char *) g_get_home_dir();
+    gtk_rc_parse(g_build_path(v->home, RCNAME, NULL));
 
     X->kbd_accel = gtk_accel_group_new();
     X->tips = gtk_tooltips_new();
     X->dpy = GDK_DISPLAY();
-    set_gcalctool_icon();
+
+    gtk_window_set_default_icon_name("gnome-calculator");
 
     do_calctool(argc, argv);
 
@@ -491,29 +502,27 @@ update_statusbar(gchar *text, const gchar *imagename)
 static void
 about_cb(GtkAction *action, gpointer date)
 {
-    
-        const gchar *authors[] = {
-            "Rich Burridge <rich.burridge@sun.com>",
-            "Sami Pietila <sampie@ariana-dsl.utu.fi>",
-            NULL
-        };
-        const gchar *documenters[] = {
-            "Sun Microsystems",
-            NULL
-        };
-        const gchar *translator_credits = _("translator_credits");
+    const gchar *authors[] = {
+        "Rich Burridge <rich.burridge@sun.com>",
+        "Sami Pietila <sampie@ariana-dsl.utu.fi>",
+        NULL
+    };
+    const gchar *documenters[] = {
+        "Sun Microsystems",
+        NULL
+    };
+    const gchar *translator_credits = _("translator-credits");
 
-        gtk_show_about_dialog(GTK_WINDOW (X->kframe),
-        					"name",_("Gcalctool"),
-        					"version", VERSION,
-        					"copyright", "(C) 2004 the Free Software Foundation",
-        					"comments", _("Calculator with financial and scientific modes."),
-        					"authors", authors,
-        					"documenters", documenters,
-        					"translator-credits", strcmp(translator_credits, "translator_credits") != 0 ? translator_credits : NULL,
-        					"logo", X->icon,
-                                                NULL);
-        					
+    gtk_show_about_dialog(GTK_WINDOW(X->kframe),
+            "name",_("Gcalctool"),
+            "version", VERSION,
+            "copyright", "(C) 2004 the Free Software Foundation",
+            "comments", _("Calculator with financial and scientific modes."),
+            "authors", authors,
+            "documenters", documenters,
+            "translator_credits", translator_credits,
+            "logo-icon-name", "gnome-calculator",
+            NULL);
 }
 
 
@@ -706,6 +715,7 @@ cfframe_response_cb(GtkDialog *dialog, gint id, gpointer data)
     GArray *entries = (GArray *) g_object_get_data(G_OBJECT(dialog), "entries");
 
     if (id == GTK_RESPONSE_HELP) {
+#ifndef DISABLE_GNOME
         GError *error = NULL;
 
         gnome_help_display_desktop(NULL, "gcalctool", "gcalctool", 
@@ -723,6 +733,7 @@ cfframe_response_cb(GtkDialog *dialog, gint id, gpointer data)
             g_error_free(error);
             error = NULL;
         }
+#endif /*DISABLE_GNOME*/
         return;
     }
 
@@ -834,7 +845,7 @@ create_cf_model(enum menu_type mtype, GtkWidget *dialog)
     for (i = 0; i < MAXCONFUN; i++) {
         n.number = i;
         if (mtype == M_CON) {
-            n.value = g_strdup(make_number(v->MPcon_vals[i], DEC, FALSE, TRUE));
+            n.value = g_strdup(make_number(v->MPcon_vals[i], DEC, TRUE));
             n.description = g_strdup(v->con_names[i]);
         } else {
             n.value       = g_strdup(v->fun_vals[i]);
@@ -1036,8 +1047,11 @@ gcalc_window_get_menu_items(XVars X)
     GtkAction *act;
     GSList *p = NULL;
     gint i = 0;
-    static gchar *action_image[] = { "Quit", "Copy", "Paste", 
-                                     "Contents", "About", NULL };
+    static const gchar *action_image[] = { "Quit", "Copy", "Paste", 
+#ifndef DISABLE_GNOME
+                                     "Contents", 
+#endif /*DISABLE_GNOME*/
+                                     "About", NULL };
 
     while (action_image[i]) {
         act = gtk_action_group_get_action (X->actions, action_image[i]);
@@ -1130,16 +1144,16 @@ create_kframe()
 
     X->actions = gtk_action_group_new("Actions");
     gtk_action_group_set_translation_domain (X->actions, NULL);
-    gtk_action_group_add_actions(X->actions, entries, n_entries, NULL);
+    gtk_action_group_add_actions(X->actions, entries, G_N_ELEMENTS(entries), NULL);
     gtk_action_group_add_toggle_actions(X->actions,
-                                        toggle_entries, n_toggle_entries,
+                                        toggle_entries, G_N_ELEMENTS(toggle_entries),
                                         NULL);
     gtk_action_group_add_radio_actions(X->actions,
-                                       acc_radio_entries, n_acc_radio_entries,
+                                       acc_radio_entries, G_N_ELEMENTS(acc_radio_entries),
                                        -1, G_CALLBACK(mb_acc_radio_proc),
                                        NULL);
     gtk_action_group_add_radio_actions(X->actions,
-                                       base_radio_entries, n_base_radio_entries,
+                                       base_radio_entries, G_N_ELEMENTS(base_radio_entries),
                                        M_BASIC, G_CALLBACK(mb_base_radio_proc),
                                        NULL);
 
@@ -1157,6 +1171,8 @@ create_kframe()
     gtk_widget_show(X->menubar);
     gtk_box_pack_start(GTK_BOX(X->kvbox), X->menubar, FALSE, FALSE, 0);
 
+    g_object_set(gtk_ui_manager_get_action(X->ui, "/MenuBar/ViewMenu/Memory"),
+                 "sensitive", (v->modetype != BASIC), NULL);
 
     X->scrolledwindow = gtk_scrolled_window_new(NULL, NULL);
     gtk_widget_show(X->scrolledwindow);
@@ -1221,7 +1237,6 @@ create_kframe()
     gtk_widget_show(X->adv_panel);
     gtk_window_add_accel_group(GTK_WINDOW(X->kframe), X->kbd_accel);
     grey_buttons(v->base);
-    gtk_window_set_icon(GTK_WINDOW(X->kframe), X->icon);
     if (v->modetype == BASIC) {
         gtk_window_set_focus(GTK_WINDOW(X->kframe), GTK_WIDGET(BUT_CLR_BAS));
     } else {
@@ -1293,10 +1308,10 @@ create_mem_menu(enum menu_type mtype)
     X->menus[(int) mtype] = gtk_menu_new();
 
     for (i = 0; i < MAXREGS; i++) {
-        SPRINTF(mstr, "<span weight=\"bold\">%s%d:</span>    %s",
+        SPRINTF(mstr, "<span weight=\"bold\">%s_%d:</span>    %s",
 	/* translators: R is the short form of register used inter alia
 	in popup menus */
-                _("R"), i, make_number(v->MPmvals[i], v->base, FALSE, TRUE));
+                _("R"), i, make_number(v->MPmvals[i], v->base, TRUE));
         create_menu_item_with_markup(mstr, m, i);
     }
 }
@@ -1421,7 +1436,6 @@ create_rframe()
     g_object_set_data(G_OBJECT(X->rframe), "rframe", X->rframe);
     gtk_window_set_resizable(GTK_WINDOW(X->rframe), TRUE);
     gtk_window_set_title(GTK_WINDOW(X->rframe), _("Memory Registers"));
-    gtk_window_set_icon(GTK_WINDOW(X->rframe), X->icon); 
 
     vbox = gtk_vbox_new(FALSE, 0);
     gtk_container_add(GTK_CONTAINER(X->rframe), vbox);
@@ -1429,7 +1443,7 @@ create_rframe()
 
     for (i = 0; i < MAXREGS; i++) {
         SPRINTF(line, "<span weight=\"bold\">%s%1d:</span>   %s", 
-                _("R"), i,  make_number(v->MPmvals[i], v->base, FALSE, TRUE));
+                _("R"), i,  make_number(v->MPmvals[i], v->base, TRUE));
         X->regs[i] = gtk_label_new("");
         gtk_label_set_markup(GTK_LABEL(X->regs[i]), line);
         SPRINTF(name, "register_label%1d", i);
@@ -1446,12 +1460,14 @@ create_rframe()
 }
 
 
+#ifndef DISABLE_GNOME
 /*ARGSUSED*/
 static void
 die_cb(GnomeClient *client, gpointer data)
 {
     gtk_main_quit();
 }
+#endif /*DISABLE_GNOME*/
 
 
 /*ARGSUSED*/
@@ -1468,6 +1484,7 @@ dismiss_aframe(GtkWidget *widget, GdkEvent *event, gpointer user_data)
 static gboolean
 dismiss_rframe(GtkWidget *widget, GdkEvent *event, gpointer user_data)
 {
+    v->rstate = 0;
     set_memory_toggle(FALSE);
     put_resource(R_REGS, "false");
     gtk_widget_hide(X->rframe);
@@ -1506,15 +1523,15 @@ create_con_fun_menu(enum menu_type mtype)
     for (i = 0; i < MAXCONFUN; i++) {
         invalid = 0;
         if (mtype == M_CON) {
-            SPRINTF(mline, "<span weight=\"bold\">%s%1d:</span> %s [%s]", 
+            SPRINTF(mline, "<span weight=\"bold\">%s_%1d:</span> %s [%s]", 
                     _("C"), i, 
-                    make_number(v->MPcon_vals[i], DEC, FALSE, TRUE), 
+                    make_number(v->MPcon_vals[i], DEC, TRUE), 
                     v->con_names[i]);
         } else {
             if (!strlen(v->fun_vals[i])) {
                 invalid = 1;
             } else {
-	      SPRINTF(mline, "<span weight=\"bold\">%s%1d:</span> %s [%s]", 
+	      SPRINTF(mline, "<span weight=\"bold\">%s_%1d:</span> %s [%s]", 
 		      _("F"), i, v->fun_vals[i], v->fun_names[i]);
             }
         }
@@ -1532,16 +1549,17 @@ create_menu_item_with_markup(char *label, int menu_no, int user_data)
     GtkWidget *accel_label;
     GtkWidget *menu_item;
 
-    accel_label = gtk_accel_label_new("");
-    gtk_label_set_markup(GTK_LABEL(accel_label), label);
-    menu_item = gtk_menu_item_new();
+    accel_label = gtk_label_new(NULL); 
+    gtk_label_set_markup_with_mnemonic(GTK_LABEL(accel_label), label); 
     gtk_misc_set_alignment(GTK_MISC(accel_label), 0.0, 0.5);
+    menu_item = gtk_menu_item_new(); 
     gtk_container_add(GTK_CONTAINER(menu_item), accel_label);
-    gtk_accel_label_set_accel_widget(GTK_ACCEL_LABEL(accel_label), menu_item);
     gtk_widget_show(accel_label);
     gtk_widget_show(menu_item);
+
     g_object_set_data(G_OBJECT(menu_item), "mtype", (gpointer) menu_no);
     gtk_menu_shell_append(GTK_MENU_SHELL(X->menus[menu_no]), menu_item);
+
     g_signal_connect(G_OBJECT(menu_item), "activate",
                      G_CALLBACK(menu_proc_cb), (gpointer) user_data);
 }
@@ -1742,6 +1760,7 @@ handle_selection()  /* Handle the GET function key being pressed. */
 static void
 help_cb()
 {
+#ifndef DISABLE_GNOME
     GError *error = NULL;
 
     gnome_help_display_desktop(NULL, "gcalctool", "gcalctool", NULL, &error);
@@ -1750,6 +1769,7 @@ help_cb()
         g_error_free(error);
         error = NULL;
     }
+#endif /*DISABLE_GNOME*/
 }
 
 
@@ -1978,8 +1998,10 @@ make_but_panel(GtkWidget *vbox, GtkWidget **Gtk_buttons,
                 Gtk_buttons[n] = make_menu_button(label, j*cols + i);
             }
             set_accessible_name(Gtk_buttons[n], buttons[n]);
-            g_signal_connect(G_OBJECT(Gtk_buttons[n]), "clicked",
-                            G_CALLBACK(button_proc), (gpointer) (j*cols + i));
+            if (buttons[n].mtype == M_NONE) {
+                g_signal_connect(G_OBJECT(Gtk_buttons[n]), "clicked",
+                             G_CALLBACK(button_proc), (gpointer) (j*cols + i));
+            }
             SPRINTF(name, "%s_button%1d", tag, n);
             gtk_widget_set_name(Gtk_buttons[n], name);
             if (buttons[n].hstr != NULL) {
@@ -2014,6 +2036,14 @@ make_reg(int n, char *str)
 }
 
 
+/*ARGSUSED*/
+void
+menu_cancel_cb(GtkMenuShell *menushell, gpointer user_data)
+{
+    v->pending = 0;
+}
+
+
 static GtkWidget *
 create_menu(enum menu_type mtype, struct button *n)
 {
@@ -2049,6 +2079,9 @@ create_menu(enum menu_type mtype, struct button *n)
     gtk_container_set_border_width(GTK_CONTAINER(X->menus[m]), 1);
     X->mrec[m] = n;
 
+    g_signal_connect(G_OBJECT(X->menus[m]), "cancel",
+                     G_CALLBACK(menu_cancel_cb), (gpointer) n);
+
     return(X->menus[m]);
 }
 
@@ -2073,14 +2106,23 @@ menu_button_button_press_cb(GtkButton *widget, gpointer data)
     GtkWidget *menu;
     GdkEventButton *event = (GdkEventButton *) gtk_get_current_event();
 
-    if (event->button != 1) {
-        return;
-    }
-
     n = (struct button *) g_object_get_data(G_OBJECT(widget), "button");
     menu = create_menu(n->mtype, n);
-    gtk_menu_popup(GTK_MENU(menu), NULL, NULL, NULL, NULL,
-                   event->button, event->time);
+
+/* If gcalctool is being driven by gok, the on-screen keyboard 
+ * assistive technology, it's possible that the event returned by 
+ * gtk_get_current_event() is NULL. If this is the case, we need 
+ * to fudge the popping up on the menu associated with this menu 
+ * button.
+ */
+
+    if (event == NULL) {
+        gtk_menu_popup(GTK_MENU(menu), NULL, NULL, NULL, NULL,
+                       1, gtk_get_current_event_time());
+    } else if (event->button == 1) {
+        gtk_menu_popup(GTK_MENU(menu), NULL, NULL, NULL, NULL,
+                       event->button, event->time);
+    }
 }
 
 
@@ -2089,18 +2131,9 @@ static gboolean
 menu_button_key_press_cb(GtkWidget *widget, 
                          GdkEventKey *event, gpointer data)
 {
-    struct button *n;
-    GdkPoint loc;
-    GtkWidget *menu;
-
     if (event->keyval == GDK_space) {
-        n = (struct button *) g_object_get_data(G_OBJECT(widget), "button");
-        menu = create_menu(n->mtype, n);
-        gdk_window_get_origin(widget->window, &loc.x, &loc.y);
-        loc.x += widget->allocation.x;
-        loc.y += widget->allocation.y;
-        gtk_menu_popup(GTK_MENU(menu), NULL, NULL, menu_pos_func,
-                       (gpointer) &loc, event->keyval, event->time);
+        show_menu_for_button(widget, event);
+
         return(TRUE);
     }
 
@@ -2163,6 +2196,7 @@ toggle_expressions()
         case exprs:
             v->e.calc_complete = 0;
             MPstr_to_num("0", DEC, v->e.ans);
+	    MPstr_to_num("0", DEC, v->e.ansbak);
             exp_del();
             show_display(v->e.ans);
             update_statusbar(
@@ -2201,11 +2235,11 @@ mb_proc(GtkAction *action)
         do_memory();
     } else if (EQUAL(name, "Contents")) {
         help_cb();
-    } else if (EQUAL(name, "LSPlaces")) {
+    } else if (!strncmp(name, "LSPlaces", 8)) {
         SSCANF(name,"LSPlaces%d", &choice);
         choice += (choice < 10) ? '0' : 'A' - 10;
         handle_menu_selection(X->mrec[(int) M_LSHF], choice);
-    } else if (EQUAL(name, "RSPlaces")) {
+    } else if (!strncmp(name, "RSPlaces", 8)) {
         SSCANF(name,"RSPlaces%d", &choice);
         choice += (choice < 10) ? '0' : 'A' - 10;
         handle_menu_selection(X->mrec[(int) M_RSHF], choice);
@@ -2272,6 +2306,8 @@ mstz_proc(GtkAction *action)
 	put_resource(R_ZEROES, set_bool(v->show_zeroes == TRUE));
 	make_registers();
     }
+
+    v->pending = 0;
 }
 
 
@@ -2359,20 +2395,18 @@ reset_mode_values(enum mode_type mtype)
     set_show_zeroes_toggle(v->show_zeroes);
     put_resource(R_ZEROES, set_bool(v->show_zeroes == TRUE));
 
+    g_object_set(gtk_ui_manager_get_action(X->ui, "/MenuBar/ViewMenu/Memory"),
+                 "sensitive", (v->modetype != BASIC), NULL);
+    if (v->modetype == BASIC) {
+        dismiss_rframe(NULL, NULL, NULL);
+    }
+
     show_display(v->MPdisp_val);
     make_registers();
     do_mode();
 }
 
 
-/*ARGUSED*/
-static void
-spframe_adj_cb(GtkAdjustment *adj, gpointer data)
-{
-    g_print("New value is %f\n", adj->value);
-}
-
-
 /*ARGSUSED*/
 static void
 spframe_cancel_cb(GtkButton *button, gpointer user_data)
@@ -2749,14 +2783,6 @@ set_title(enum fcp_type fcptype, char *str)
 }
 
 
-static void
-set_gcalctool_icon(void)
-{    
-    X->icon = gtk_icon_theme_load_icon(gtk_icon_theme_get_default(),
-                                       "gnome-calculator", 48, 0, NULL);
-}
-
-
 static void
 set_show_tsep_toggle(int state)
 {
@@ -2800,6 +2826,51 @@ show_ascii_frame()      /* Display ASCII popup. */
 }
 
 
+void
+show_menu()
+{
+    GtkWidget *button = NULL;
+    GdkEvent *event = gtk_get_current_event();
+
+       if (key_equal(v->current, KEY_ACC)) {              /* Acc */
+        button = BUT_ACC;
+    } else if (key_equal(v->current, KEY_CON)) {          /* Con */
+        button = BUT_CON;
+    } else if (key_equal(v->current, KEY_EXCH)) {         /* Exch */
+        button = BUT_EXCH;
+    } else if (key_equal(v->current, KEY_FUN)) {          /* Fun */
+        button = BUT_FUN;
+    } else if (key_equal(v->current, KEY_LSFT)) {         /* < */
+        button = BUT_LSFT;
+    } else if (key_equal(v->current, KEY_RCL)) {          /* Rcl */
+        button = BUT_RCL;
+    } else if (key_equal(v->current, KEY_RSFT)) {         /* > */
+        button = BUT_RSFT;
+    } else if (key_equal(v->current, KEY_STO)) {          /* Sto */
+        button = BUT_STO;
+    }
+
+    show_menu_for_button(button, (GdkEventKey *) event);
+}
+
+
+static void
+show_menu_for_button(GtkWidget *widget, GdkEventKey *event)
+{
+    struct button *n;
+    GdkPoint loc;
+    GtkWidget *menu;
+
+    n = (struct button *) g_object_get_data(G_OBJECT(widget), "button");
+    menu = create_menu(n->mtype, n);
+    gdk_window_get_origin(widget->window, &loc.x, &loc.y);
+    loc.x += widget->allocation.x;
+    loc.y += widget->allocation.y;
+    gtk_menu_popup(GTK_MENU(menu), NULL, NULL, menu_pos_func,
+                   (gpointer) &loc, event->keyval, event->time);
+}
+
+
 static void
 show_precision_frame()      /* Display Set Precision popup. */
 {
@@ -2832,6 +2903,24 @@ start_tool()
     set_item(NUMITEM, v->dtype);
 
     gtk_widget_show(X->kframe);
+
+    switch (v->syntax) { 
+        case npa:
+	    break;
+        case exprs:
+	    // Init expression mode.
+	    // This must be executed after do_base is called at init.
+	    // FIXME: The init code here is duplicated elsewhere
+	    v->e.calc_complete = 0;
+	    MPstr_to_num("0", DEC, v->e.ans);
+	    MPstr_to_num("0", DEC, v->e.ansbak);
+            exp_del();
+            show_display(v->e.ans);
+	    break;
+    default:
+	assert(0);
+    }
+
     gtk_main();
 }
 
@@ -2868,7 +2957,6 @@ win_display(enum fcp_type fcptype, int state)
     }
     if (state) {
         if (fcptype == FCP_REG) {
-            set_memory_toggle(v->rstate);
             ds_position_popup(X->kframe, f, DS_POPUP_ABOVE);
         }
     }
